This patch adds the ability to disable Address Translation Services (ATS) by
passing pci=noats as a kernel boot parameter.

diff --git a/drivers/pci/ats.c b/drivers/pci/ats.c
index eeb9fb2..2930cd7 100644
--- a/drivers/pci/ats.c
+++ b/drivers/pci/ats.c
@@ -17,10 +17,20 @@
 
 #include "pci.h"
 
+static int pci_ats_disable;
+
+void pci_no_ats(void)
+{
+	pci_ats_disable = 1;
+}
+
 void pci_ats_init(struct pci_dev *dev)
 {
 	int pos;
 
+	if (pci_ats_disable)
+		return;
+
 	pos = pci_find_ext_capability(dev, PCI_EXT_CAP_ID_ATS);
 	if (!pos)
 		return;
diff --git a/drivers/pci/pci.c b/drivers/pci/pci.c
index ba34907..e7dd3e2 100644
--- a/drivers/pci/pci.c
+++ b/drivers/pci/pci.c
@@ -5297,6 +5297,8 @@ static int __init pci_setup(char *str)
 				pci_no_msi();
 			} else if (!strcmp(str, "noaer")) {
 				pci_no_aer();
+			} else if (!strcmp(str, "noats")) {
+				pci_no_ats();
 			} else if (!strncmp(str, "realloc=", 8)) {
 				pci_realloc_get_opt(str + 8);
 			} else if (!strncmp(str, "realloc", 7)) {
diff --git a/include/linux/pci.h b/include/linux/pci.h
index 0e49f70..40f107e 100644
--- a/include/linux/pci.h
+++ b/include/linux/pci.h
@@ -1406,11 +1406,13 @@ void ht_destroy_irq(unsigned int irq);
 
 #ifdef CONFIG_PCI_ATS
 /* Address Translation Service */
+void pci_no_ats(void);
 void pci_ats_init(struct pci_dev *dev);
 int pci_enable_ats(struct pci_dev *dev, int ps);
 void pci_disable_ats(struct pci_dev *dev);
 int pci_ats_queue_depth(struct pci_dev *dev);
 #else
+static inline void pci_no_ats(void) { }
 static inline void pci_ats_init(struct pci_dev *d) { }
 static inline int pci_enable_ats(struct pci_dev *d, int ps) { return -ENODEV; }
 static inline void pci_disable_ats(struct pci_dev *d) { }
